#include <QtGui>
#include <QWidget>
#include <QPushButton>

#include "window.h"

Window::Window()
     : QWidget()
{

}

Main::Main() : QWidget() {
		QResource::registerResource("/scratchbox/users/figa/home/figa/workspace/sudoku/src/resources.rcc");
		master = new QWidget();
                //master->setAttribute(Qt::WA_Maemo5StackedWindow);
		slave = new QWidget(master);
//		slave->setAttribute(Qt::WA_Maemo5StackedWindow);
		slave->setWindowFlags(slave->windowFlags() | Qt::Window);
		running = false;


		master->setWindowTitle(tr("fSudoku::Menu"));
		QLabel * logo= new QLabel(this);
		logo->setPixmap(QPixmap(":/logo.png"));
		QPushButton * newGame = new QPushButton(tr("New Game"));
		QPushButton * continueGame = new QPushButton(tr("Continue"));
		QPushButton * customGame = new QPushButton(tr("Custom"));
		QGridLayout * menu = new QGridLayout();
		level = new QComboBox();
		level->addItem("Easy");
		level->addItem("Normal");
		level->addItem("Hard");
		level->addItem("Extreme");
		menu->addWidget(logo,0,0,1,2,Qt::AlignCenter);
		menu->addWidget(newGame,1,0);
		menu->addWidget(level,1,1);
		menu->addWidget(continueGame,2,0);
		menu->addWidget(customGame,2,1);
		master->setLayout(menu);
		connect(newGame, SIGNAL(clicked()), this, SLOT(newGame()));
		level->connect(level, SIGNAL(currentIndexChanged(int)), this, SLOT(setLevel()));
		connect(continueGame, SIGNAL(clicked()), this, SLOT(loadGame()));
		connect(customGame, SIGNAL(clicked()), this, SLOT(customGame()));


		timeTimer = 0;
		timer = new QTimer(this);
		connect(timer, SIGNAL(timeout()), this, SLOT(updateTimer()));
		mainLayout = new QHBoxLayout();
		QHBoxLayout *unfilledCounter = new QHBoxLayout;
		QVBoxLayout *leftLayout = new QVBoxLayout;
		timeLabel = new QLabel("0");
		QLabel *unfilledCounterText = new QLabel(tr("Left:"));

		//set fonts [INIT]
		defaultFont = timeLabel->font();
		candidateFont.setPointSize(defaultFont.pointSize()/2);

		slave->setWindowTitle(tr("fSudoku::Game"));


		sprintf(cacheCharSmall,"%i",fSudo.unfilled);
		unfilledCounterLabel = new QLabel(cacheCharSmall);
		unfilledCounter->addWidget(unfilledCounterText);
		unfilledCounter->addWidget(unfilledCounterLabel);
		leftLayout->addWidget(timeLabel);
		leftLayout->addLayout(unfilledCounter);
		mainLayout->addLayout(leftLayout);
		slave->setLayout(mainLayout);
		timer->start(1000);
		master->show();

}

void Main::closeEvent(QCloseEvent *event) {
	fSudo.save();
}

void Main::setGameField(int mode) {
	QSpacerItem *spacer = new QSpacerItem(4,4);
	signalMapperGrid = new QSignalMapper(this);
	grid = new QGridLayout();
	grid->setSpacing(0);
	if(mode==0) {
		fSudo.newSudoku();
		fSudo.save();
	}
	else if (mode==1) {
		fSudo.load();
	}
	for(int x=0;x<9;x++) {
		for(int y=0;y<9;y++) {
			char buffer[2];
			sprintf(buffer,"%i",fSudo.unsolved[x][y]);
			if(buffer[0]=='0' || mode==2) {
				if(fSudo.candidates[x][y][0]=='0') {
					elements[x][y] = new QPushButton(tr(""), this);
					fSudo.unsolved[x][y]=0;
					fSudo.base[x][y]=0;
				}
				else {
					//candidates
					elements[x][y] = new QPushButton(fSudo.candidates[x][y].c_str(), this);
					elements[x][y]->setFont(candidateFont);
				}
				elements[x][y]->setFixedSize(45,45);
			}
			else {
				elements[x][y] = new QPushButton(tr(buffer), this);
				if(fSudo.unsolved[x][y] == fSudo.base[x][y]) {
					elements[x][y]->setEnabled(FALSE);
				}
				elements[x][y]->setFixedSize(45,45);
				--fSudo.unfilled;
			}
			elements[x][y]->setCheckable(true);
			if(x<3) {
				if(y<3) {
					grid->addWidget(elements[x][y], x, y);
				}
				else if(y<6) {
					grid->addWidget(elements[x][y], x, y+1);
				}
				else {
					grid->addWidget(elements[x][y], x, y+2);
				}
			}
			else if(x<6){
				if(y<3) {
					grid->addWidget(elements[x][y], x+1, y);
				}
				else if(y<6) {
					grid->addWidget(elements[x][y], x+1, y+1);
				}
				else {
					grid->addWidget(elements[x][y], x+1, y+2);
				}
			}
			else {
				if(y<3) {
					grid->addWidget(elements[x][y], x+2, y);
				}
				else if(y<6) {
					grid->addWidget(elements[x][y], x+2, y+1);
				}
				else {
					grid->addWidget(elements[x][y], x+2, y+2);
				}
			}
			connect(elements[x][y], SIGNAL(clicked()), signalMapperGrid, SLOT(map()));
			signalMapperGrid->setMapping(elements[x][y], elements[x][y]);
		}
	}
	//update unfilled counter
	sprintf(cacheCharSmall,"%i",fSudo.unfilled);
	unfilledCounterLabel->setText(cacheCharSmall);
	grid->addItem(spacer,3,0);
	grid->addItem(spacer,7,0);
	grid->addItem(spacer,0,3);
	grid->addItem(spacer,0,7);
	connect(signalMapperGrid, SIGNAL(mapped(QWidget*)),this, SLOT(oneChecked(QWidget*)));
	mainLayout->addLayout(grid);
}

void Main::setControl() {
	signalMapper = new QSignalMapper(this);
	touchControl = new QGridLayout();
	undoButton = new QPushButton(this);
	undoButton->setIcon(QIcon(":/undo.png"));
	undoButton->setEnabled(FALSE);
	redoButton = new QPushButton(this);
	redoButton->setIcon(QIcon(":/redo.png"));
	redoButton->setEnabled(FALSE);
	solveButton = new QPushButton(this);
	solveButton->setIcon(QIcon(":/solve.png"));
	candidateButton = new QPushButton(this);
	candidateButton->setIcon(QIcon(":/candidate.png"));
	candidateButton->setCheckable(TRUE);
	touchControl->setSpacing(0);
	//slots
	connect(undoButton, SIGNAL(clicked()), this, SLOT(undo()));
	connect(redoButton, SIGNAL(clicked()), this, SLOT(redo()));
	connect(solveButton, SIGNAL(clicked()), this, SLOT(solve()));
	connect(candidateButton, SIGNAL(clicked()), this, SLOT(addCandidate()));
	for(int i=0; i<10; i++) {
				char buffer[2];
				sprintf(buffer,"%i",i);
				control[i] = new QPushButton(tr(buffer), this);
				connect(control[i], SIGNAL(clicked()), signalMapper, SLOT(map()));
				signalMapper->setMapping(control[i], i);
				if(i==0) {
					touchControl->addWidget(control[i],0,0);
				}
				else if(i==1) {
					touchControl->addWidget(control[i],0,1);
				}
				else if(i==2) {
					touchControl->addWidget(control[i],1,0);
				}
				else if(i==3){
					touchControl->addWidget(control[i],1,1);;
				}
				else if(i==4) {
					touchControl->addWidget(control[i],2,0);
				}
				else if(i==5) {
					touchControl->addWidget(control[i],2,1);
				}
				else if(i==6) {
					touchControl->addWidget(control[i],3,0);
				}
				else if(i==7) {
					touchControl->addWidget(control[i],3,1);
				}
				else if(i==8){
					touchControl->addWidget(control[i],4,0);;
				}
				else {
					touchControl->addWidget(control[i],4,1);
				}
			}

			connect(signalMapper, SIGNAL(mapped(int)),this, SLOT(setElement(int)));
			control[0]->setText("Del");
			control[0]->setStyleSheet("* { color: #0000ff }");
			touchControl->addWidget(candidateButton,5,0);
			touchControl->addWidget(solveButton,5,1);
			touchControl->addWidget(undoButton,6,0);
			touchControl->addWidget(redoButton,6,1);
			mainLayout->addLayout(touchControl);
}

void Main::handlePressed()
{
    QMessageBox::information(this, tr("Info"), tr("Hello World!!!"));
}

void Main::newGame() {
	if(running) {
		mainLayout->removeItem(touchControl);
		mainLayout->removeItem(grid);
		fSudo.unfilled=81;
	}
	setGameField(0);
	setControl();
	slave->show();
	running = true;
}

void Main::setLevel() {
	if(level->currentIndex()==0) {
		fSudo.level=45;
	}
	else if (level->currentIndex()==1) {
		fSudo.level=35;
	}
	else if(level->currentIndex()==2) {
		fSudo.level=30;
	}
	else {
		fSudo.level=25;
	}
}

void Main::loadGame() {
	if(running) {
		mainLayout->removeItem(touchControl);
		mainLayout->removeItem(grid);
		fSudo.unfilled=81;
	}
	setGameField(1);
	setControl();
	slave->show();
	running = true;
}

void Main::customGame() {
	memcpy(fSudo.base,fSudo.unsolved,sizeof(fSudo.unsolved));
	if(running) {
		mainLayout->removeItem(touchControl);
		mainLayout->removeItem(grid);
		fSudo.unfilled=81;
	}
	setGameField(2);
	setControl();
	slave->show();
	running=true;
}

void Main::setElement(int value) {
	//save values for undo action
	bool changed = 0;
	char buffer[2];
    for(int x=0;x<9;x++) {
    	for(int y=0;y<9;y++) {
    		if(elements[x][y]->isChecked()) {
    			saveUndo(x,y,fSudo.unsolved[x][y],elements[x][y]->styleSheet(), elements[x][y]->font(), fSudo.candidates[x][y], fSudo.unfilled);
    			//add value to grid
    			if(!candidateButton->isChecked()) {
    				elements[x][y]->setStyleSheet("* { color: #ffffff }");
					changed = true;
					sprintf(buffer,"%i",value);
					if(buffer[0]=='0') {
						elements[x][y]->setText("");
						if(fSudo.unsolved[x][y] != 0){fSudo.unfilled++;}//only after change
						fSudo.unsolved[x][y]=value;
						fSudo.candidates[x][y]='0';
					}
					else {
						elements[x][y]->setText(buffer);
						if(fSudo.unsolved[x][y]==0) {fSudo.unfilled--;}
						fSudo.unsolved[x][y]=value;
		   				if(!fSudo.isValid(x,y)) {
		   					elements[x][y]->setStyleSheet("* { color: #ff0000 }");
		   				}
					}
				elements[x][y]->setFont(defaultFont);
    			sprintf(cacheCharSmall,"%i",fSudo.unfilled);
    			unfilledCounterLabel->setText(cacheCharSmall);
    		}
    		//add candidate to grid
    		else {
    			if(value!=0) {
    				changed = true;
    				elements[x][y]->setFont(candidateFont);
					QString  origin;
					QString newStr;
					origin = elements[x][y]->text();
					newStr.setNum(value);
					candidateButton->setChecked(FALSE);
					origin.operator +=(newStr);
					elements[x][y]->setText(origin);
					//save candidates to string
					fSudo.candidates[x][y] = origin.toStdString();
    			}
    			}
    			break;
    		}
    	}
    }
    //available if was changed value
    if(changed) {
		redoButton->setEnabled(FALSE);
		undoButton->setEnabled(TRUE);
	}
}

void Main::oneChecked(QWidget* element) {
	QPushButton *bt = qobject_cast<QPushButton *>( element );
	Q_ASSERT( bt );
	for(int x=0;x<9;x++) {
	    for(int y=0;y<9;y++) {
	    	if(elements[x][y] != bt) {
	    		elements[x][y]->setChecked(FALSE);
			}
	    }
	}
}

void Main::addCandidate() {

}

void Main::saveUndo(int x, int y, char value, QString color, QFont font, string candidates, int unfilled) {
	fSudo.undo->x = x;
	fSudo.undo->y = y;
	fSudo.undo->value[0] = value;
	fSudo.undo->value[1] = '\0';
	fSudo.undo->color = color;
	fSudo.undo->font = font;
	fSudo.undo->candidates = candidates;
	fSudo.undo->unfilled = unfilled;
}

void Main::saveRedo(int x, int y, char value, QString color, QFont font, string candidates, int unfilled) {
	fSudo.redo->x = x;
	fSudo.redo->y = y;
	fSudo.redo->value[0] = value;
	fSudo.redo->value[1] = '\0';
	fSudo.redo->color = color;
	fSudo.redo->font = font;
	fSudo.redo->candidates = candidates;
	fSudo.redo->unfilled = unfilled;
}

void Main::undo() {
	saveRedo(fSudo.undo->x, fSudo.undo->y,fSudo.unsolved[fSudo.undo->x][fSudo.undo->y],elements[fSudo.undo->x][fSudo.undo->y]->styleSheet(), elements[fSudo.undo->x][fSudo.undo->y]->font(), fSudo.candidates[fSudo.undo->x][fSudo.undo->y], fSudo.unfilled);
	if(fSudo.undo->value[0] == 0) {
		if(fSudo.undo->candidates[0] != '0') {
			QString newStr;
			int value = atoi(fSudo.undo->candidates.c_str());
			newStr.setNum(value);
			elements[fSudo.undo->x][fSudo.undo->y]->setText(newStr);
			fSudo.candidates[fSudo.undo->x][fSudo.undo->y] = fSudo.undo->candidates;
		}
		else {
			fSudo.unsolved[fSudo.undo->x][fSudo.undo->y] = '0';
			elements[fSudo.undo->x][fSudo.undo->y]->setText("");
		}
	}
	else {
		char buffer[2];
		sprintf(buffer,"%i",fSudo.undo->value[0]);
		fSudo.unsolved[fSudo.undo->x][fSudo.undo->y] = fSudo.undo->value[0];
		elements[fSudo.undo->x][fSudo.undo->y]->setText(buffer);
	}
	elements[fSudo.undo->x][fSudo.undo->y]->setStyleSheet(fSudo.undo->color);
	elements[fSudo.undo->x][fSudo.undo->y]->setFont(fSudo.undo->font);
	sprintf(cacheCharSmall,"%i",fSudo.undo->unfilled);
	unfilledCounterLabel->setText(cacheCharSmall);
    redoButton->setEnabled(TRUE);
    undoButton->setEnabled(FALSE);

    //vynulovani struktury
   fSudo.nullUndo();
}

void Main::redo() {
	saveUndo(fSudo.redo->x, fSudo.redo->y,fSudo.unsolved[fSudo.redo->x][fSudo.redo->y],elements[fSudo.redo->x][fSudo.redo->y]->styleSheet(), elements[fSudo.redo->x][fSudo.redo->y]->font(), fSudo.candidates[fSudo.redo->x][fSudo.redo->y], fSudo.unfilled);
	if(fSudo.redo->value[0] == 0) {
		if(fSudo.redo->candidates[0] != '0') {
			QString newStr;
			int value = atoi(fSudo.redo->candidates.c_str());
			newStr.setNum(value);
			elements[fSudo.redo->x][fSudo.redo->y]->setText(newStr);
			fSudo.candidates[fSudo.redo->x][fSudo.redo->y] = fSudo.redo->candidates;
		}
		else {
			fSudo.unsolved[fSudo.redo->x][fSudo.redo->y] = '0';
			elements[fSudo.redo->x][fSudo.redo->y]->setText("");
		}
	}
	else {
		char buffer[2];
		sprintf(buffer,"%i",fSudo.redo->value[0]);
		fSudo.unsolved[fSudo.redo->x][fSudo.redo->y] = fSudo.redo->value[0];
		elements[fSudo.redo->x][fSudo.redo->y]->setText(buffer);
	}
	elements[fSudo.redo->x][fSudo.redo->y]->setStyleSheet(fSudo.redo->color);
	elements[fSudo.redo->x][fSudo.redo->y]->setFont(fSudo.redo->font);
	sprintf(cacheCharSmall,"%i",fSudo.redo->unfilled);
	unfilledCounterLabel->setText(cacheCharSmall);
    redoButton->setEnabled(FALSE);
    undoButton->setEnabled(TRUE);
    //vynulovani struktury
    fSudo.nullRedo();
}

void Main::solve() {
	//set Message box
	 QMessageBox msgBox;
	 msgBox.setWindowTitle(tr("Solve Game"));
	 msgBox.setText("Do you want solve this game?");
	 msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
	 msgBox.setDefaultButton(QMessageBox::Save);
	 int ret = msgBox.exec();
	 // if yes solve game
	 switch (ret) {
	   case QMessageBox::Yes:
		   if(fSudo.unfilled<65) {
			   memcpy(fSudo.base,fSudo.unsolved,sizeof(fSudo.unsolved));
			   fSudo.save();
			   fSudo.solve();
			   char buffer[2];
			   for(int x=0;x<9;x++) {
				   for(int y=0;y<9;y++) {
					   sprintf(buffer,"%i",fSudo.solved[x][y]);
					   if(fSudo.unsolved[x][y] == fSudo.solved[x][y] && elements[x][y]->isEnabled()) {
						   elements[x][y]->setStyleSheet("* { color: #00ff00 }");
						   fSudo.unsolved[x][y]=fSudo.solved[x][y];
						   elements[x][y]->setText(buffer);
					   }
					   if(fSudo.unsolved[x][y] != fSudo.solved[x][y] && elements[x][y]->isEnabled()) {
						   elements[x][y]->setStyleSheet("* { color: #ff0000 }");
						   fSudo.unsolved[x][y]=fSudo.solved[x][y];
						   elements[x][y]->setText(buffer);
					   }
				   }
			   }
		       //stop timeTmer
		       timer->stop();
		       //set control non-active
		       for(int i = 0; i<10; i++) {
		    	   control[i]->setEnabled(FALSE);
		       }
		       //set unfilled to zero
		       fSudo.unfilled = 0;
		       unfilledCounterLabel->setText("0");
		       undoButton->setEnabled(FALSE);
		       redoButton->setEnabled(FALSE);
		       solveButton->setEnabled(FALSE);
		       candidateButton->setEnabled(FALSE);
		   }
		   else {
			   QMessageBox::information(this, tr("Solve Game"), tr("I can't solve this game. You must fill minimal 17 elements!"));
		   }
	       break;
	   default:
	       break;
	 }
}

void Main::updateTimer()
{
	timeTimer++;
		double result=timeTimer/60;
		double hour;
		double min;
		double sec;
		if(result<1) {
			hour = 0;
			sec = timeTimer;
			min = 0;
		}
		else if (result < 60) {
			hour = 0;
			min = floor(result);
			sec = (timeTimer-(min*60));
		}
		else {
			hour = floor(result/60);
			min = floor(result-(hour*60));
			sec = (timeTimer-(min*60)-(hour*60*60));
		}
		ostringstream s1;
		if((int)sec<10) {
			if(min<10) {
				s1 << hour << ":" << "0" << min << ":" << "0" << sec;
			}
			else {
				s1 << hour << ":" << min << ":" << "0" << sec;
			}
		}
		else {
			if(min<10) {
				s1 << hour << ":" << "0" << min << ":" << sec;
			}
			else {
				s1 << hour << ":" << min << ":" << sec;
			}
		}
		char text[20];
		strcpy (text,s1.str().c_str());
		timeLabel->setText(text);
}

fSudoku::fSudoku() {
	unfilled = 81;
	level = 45;
	for(int x=0;x<9;x++) {
		for(int y=0;y<9;y++) {
			candidates[x][y] = '0';
		}
	}
	/*undo = new step;
	redo = new step;
	undo->candidates = '0';*/
}

void fSudoku::newSudoku() {
	a.random_generate(level);
	//nevyresene sudoku
	memcpy(unsolved,a.data,sizeof(a.data));
	memcpy(base,a.data,sizeof(a.data));
}

void fSudoku::save() {
	FILE *file = fopen("save.txt","w");
	for (int x = 0; x < 9; x++) {
		for (int y = 0; y < 9; y++) {
			fprintf(file,"%c",unsolved[x][y]);
		}
	}
	for (int x = 0; x < 9; x++) {
		for (int y = 0; y < 9; y++) {
			fprintf(file,"%c",base[x][y]);
		}
	}
	fclose(file);
	FILE *file2 = fopen("candidates.txt","w");
	//save candidates
	char buffer[11];
	string str;
	for (int x = 0; x < 9; x++) {
		for (int y = 0; y < 9; y++) {
			str = candidates[x][y]+';';
			strcpy(buffer,str.c_str());
			fprintf(file,"%s",buffer);
		}
	}
	fclose(file2);
	//fprintf(file,"%i","5");

}

void fSudoku::load() {
	char loadUnsolved[82];
	char loadBase[82];
	char *buffer = new char[811];
	char *final = NULL;
	FILE * file;
	file = fopen ( "save.txt" , "r" );
	//load unsolved string
	fread (loadUnsolved,1,81,file);
	//load base string
	fread (loadBase,82,162,file);
	loadUnsolved[81] = '\0';
	loadBase[81] = '\0';
	FILE *file2 = fopen("candidates.txt","r");
	fread (buffer,1,810,file2);
	final = strtok (buffer, ";");
	int x=0;
	int y=0;
	while( final != NULL ) {
		string str(final);
		candidates[x][y]=str;
	    final = strtok( NULL, ";" );
	    y++;
	    if(y >= 9) {
	          x++;
	          if (x >= 9) break;
	          y = 0;
	    }
	}
	memcpy(unsolved,loadUnsolved,sizeof(loadUnsolved));
	memcpy(base,loadBase,sizeof(loadBase));
	fclose (file);
	fclose (file2);
	delete [] buffer;
}

void fSudoku::solve() {
	memcpy(a.data,unsolved,sizeof(unsolved));
	a.solve();
	memcpy(solved,a.data,sizeof(a.data));
}

bool fSudoku::isValid(int x, int y) {
	bool valid = true;
	int n =0;
	int m = 0;
	if(x<3 && y<3) {
		for(n=0;n<3;n++) {
			for(m=0;m<3;m++) {
				if(unsolved[x][y]==unsolved[n][m] && x!=n && y!=m) {
					valid = false;
					if(!valid){break;}
				}
			}
			if(!valid){break;}
		}
	}
	else if(x<3 && y<6) {
		for(n=0;n<3;n++) {
			for(m=3;m<6;m++) {
				if(unsolved[x][y]==unsolved[n][m] && x!=n && y!=m) {
					valid = false;
					if(!valid){break;}
				}
			}
			if(!valid){break;}
		}
	}
	else if(x<3 && y<9) {
		for(n=0;n<3;n++) {
			for(m=0;m<9;m++) {
				if(unsolved[x][y]==unsolved[n][m] && x!=n && y!=m) {
					valid = false;
					if(!valid){break;}
				}
			}
			if(!valid){break;}
		}
	}
	else if(x<6 && y<3) {
		for(n=3;n<6;n++) {
			for(m=0;m<3;m++) {
				if(unsolved[x][y]==unsolved[n][m] && x!=n && y!=m) {
					valid = false;
					if(!valid){break;}
				}
			}
			if(!valid){break;}
		}
	}
	else if(x<6 && y<6) {
		for(n=3;n<6;n++) {
			for(m=3;m<6;m++) {
				if(unsolved[x][y]==unsolved[n][m] && x!=n && y!=m) {
					valid = false;
					if(!valid){break;}
				}
			}
			if(!valid){break;}
		}
	}
	else if(x<6 && y<9) {
		for(n=3;n<6;n++) {
			for(m=6;m<9;m++) {
				if(unsolved[x][y]==unsolved[n][m] && x!=n && y!=m) {
					valid = false;
					if(!valid){break;}
				}
			}
			if(!valid){break;}
		}
	}
	else if(x<9 && y<3) {
		for(n=6;n<9;n++) {
			for(m=0;m<3;m++) {
				if(unsolved[x][y]==unsolved[n][m] && x!=n && y!=m) {
					valid = false;
					if(!valid){break;}
				}
			}
			if(!valid){break;}
		}
	}
	else if(x<9 && y<6) {
		for(n=6;n<9;n++) {
			for(m=3;m<6;m++) {
				if(unsolved[x][y]==unsolved[n][m] && x!=n && y!=m) {
					valid = false;
					if(!valid){break;}
				}
			}
			if(!valid){break;}
		}
	}
	else {
		for(n=6;n<9;n++) {
			for(m=6;m<9;m++) {
				if(unsolved[x][y]==unsolved[n][m] && x!=n && y!=m) {
					valid = false;
					if(!valid){break;}
				}
			}
			if(!valid){break;}
		}
	}

	for(n=0;n<9;n++) {
		if(unsolved[x][y]==unsolved[x][n] && y != n) {
			valid = false;
			if(!valid){break;}
		}
		if(unsolved[x][y]==unsolved[n][y] && x != n) {
			valid = false;
			if(!valid){break;}
		}

	}
	return valid;
}

void fSudoku::nullUndo() {
	delete undo;
	undo = new step;

}

void fSudoku::nullRedo() {
	delete redo;
	redo = new step;

}
